"""
    Declare table element class.
"""
import zLOG
import string
from AbstractElement import AbstractElement
from AccessControl import ClassSecurityInfo
from Article import Article
import Globals, interfaces

from Products.CMFCore import CMFCorePermissions, utils
import ArticlePermissions
from utils import guessFormat, handleText

class TableElement(AbstractElement):
    """
    Represent a table with a variable number of rows and columns.
    """
    __implements__ = (interfaces.Element,)

    meta_type = 'Table Element'
    security = ClassSecurityInfo()

    _editableAttributes = ('rows', 'cols', 'text', 'html',)
    _editableExtraAttributes = ('clear', 'render_as',)

    def __init__(self, id):
        self.id = id
        self.size = 0
        self.rows = 1
        self.cols = 1
        self.text = ['']
        self.html = ['']
        self.render_as = 'stx'

    security.declareProtected(ArticlePermissions.ChangeArticles, 'getEditForm')
    def getEditForm(self):
        " Return the skinned name of the edit form "
        return 'atable_edit'

    security.declareProtected(ArticlePermissions.ChangeArticles, 'getEditExtraForm')
    def getEditExtraForm(self):
        """ Return the skinned name of the edit form """
        return 'atable_edit_extra'

    security.declareProtected(CMFCorePermissions.View, 'getView')
    def getView(self):
        " Return the skinned name of the edit form "
        return 'atable_view'

    security.declareProtected(CMFCorePermissions.View, 'getCookedCell')
    def getCookedCell(self, cell):
        " Return the formatted content of cell "
        try:
            if len(self.html) > cell:
                return self.html[cell]
            else:
                return ''
        except:
            return self.getRawCell(cell)

    security.declareProtected(CMFCorePermissions.View, 'getRawCell')
    def getRawCell(self, cell):
        " Return the raw content of cell "
        if len(self.text) > cell:
            return self.text[cell]
        else:
            return ''

    security.declareProtected(ArticlePermissions.ChangeArticles, 'edit')
    def edit(self, rows, cols, text, html):
        """ Update the value """
        self.text = []
        self.html = []
        if text is not None:
            if type(text) == type(''):
                self.text.append(text)
                self.size = len(text)
            elif type(text) == type([]):
                size = 0
                for str in text:
                    headers, body, format = handleText(text=str)
                    self.text.append(body)
                    if format == 'stx':
                        body = utils._format_stx(str)
                        self.html.append(body)
                    size = size + len(body)
            #for str in self.text:
            #    self.html.append(utils._format_stx(str))
        else:
            self.text = text
            self.html = text
            size = 0
        self.size = size
        self.rows = rows
        self.cols = cols

    security.declareProtected(ArticlePermissions.ChangeArticles, 'edit')
    def editExtra(self, clear, render_as):
        """ Update the value """
        self.clear = str(clear)
        self.render_as = str(render_as)
        if self.render_as == 'stx':
            self.size = len(string.join(self.html, ' '))
        else:
            self.size = len(string.join(self.text, ' '))

#    security.declareProtected(CMFCorePermissions.View, 'getElementItems')
#    def getElementItems( self ):
#        """ Return a sequence of element items, used build the article """
#        return ((self.rows, self.cols, self.text),)

    security.declareProtected(CMFCorePermissions.View, 'manage_FTPget')
    def manage_FTPget( self ):
        """ Return the text or html """
        return self.read()

    def read(self):
        """
        return either the raw text or the formatted text
        """
        if self.render_as == 'stx':
            return string.join(self.html, ' ')
        else:
            return string.join(self.text, ' ')

    def get_size(self):
        """
        return the size of the rendered value
        """
        return self.size

    def SearchableText(self):
        """
            SeachableText is used for full text seraches of a portal.
            It should return a concatanation of all useful text.
        """
        return "%s %s" % (string.join(self.text, ' '), self.Description())

    def manage_afterAdd(self, item, container):
        """AbstractElement has an afterAdd method"""
        zLOG.LOG(self.meta_type, zLOG.INFO, 'manage_afterAdd(%s, %s)' % (item.getId(), container.getId()))
        AbstractElement.manage_afterAdd(self, item, container)


Globals.InitializeClass(TableElement)

# Register as an element type with the Article class
Article._elementTypes.append(TableElement)
