"""
    Declare text element class.
"""
import zLOG
from AbstractElement import AbstractElement
from AccessControl import ClassSecurityInfo
from Article import Article
import Globals, interfaces
from Products.CMFCore import utils
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.PortalContent import NoWL, ResourceLockedError
import ArticlePermissions
from utils import guessFormat, handleText

class TextElement(AbstractElement):
    """
    Structured text.
    """
    #__implements__ = (interfaces.Element,)

    render_as = 'stx'
    size = 0
    meta_type = 'Text Element'
    _stx_level = 1                      # Structured text level
    content_types = ('text/plain','text/html')

    security = ClassSecurityInfo()

    _editableAttributes = ('value', 'html',)
    _editableExtraAttributes = ('clear', 'alignment', 'render_as')

    def __init__(self, id):
        """
        Constructor
        """
        zLOG.LOG(self.meta_type, zLOG.INFO, '__init__(%s)' % id)
        self.id = id

    security.declareProtected(ArticlePermissions.ChangeArticles, 'getEditForm')
    def getEditForm(self):
        " Return the skinned name of the edit form "
        return 'atext_edit'

    security.declareProtected(ArticlePermissions.ChangeArticles, 'getEditExtraForm')
    def getEditExtraForm(self):
        """ Return the skinned name of the edit form """
        return 'atext_edit_extra'

    security.declareProtected(CMFCorePermissions.View, 'getView')
    def getView(self):
        " Return the skinned name of the edit form "
        return 'atext_view'

    def PUT(self, REQUEST, RESPONSE):
        """ Handle HTTP (and presumably FTP?) PUT requests.
            (Copied from CMFDefault.Document.py)
        """
        zLOG.LOG(self.meta_type, zLOG.INFO, 'PUT')
        if not NoWL:
            self.dav__init(REQUEST, RESPONSE)
            self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        body = REQUEST.get('BODY', '')
        zLOG.LOG(self.meta_type, zLOG.INFO, 'PUT: body=%s...' % body[1:15])
        #guessedformat = REQUEST.get_header('Content-Type', 'text/plain')
        #ishtml = (guessedformat == 'text/html') or utils.html_headcheck(body)
        #zLOG.LOG(self.meta_type, zLOG.INFO, 'PUT')

        #if ishtml: self.render_as = 'html'
        #else: self.render_as = 'stx'

        try:
            self.edit(value=body, html='')
        except ResourceLockedError, msg:
            zLOG.LOG(self.meta_type, zLOG.INFO, 'PUT: %s...' % msg)
            get_transaction().abort()
            RESPONSE.setStatus(423)
            return RESPONSE

        RESPONSE.setStatus(204)
        return RESPONSE

    security.declareProtected(ArticlePermissions.ChangeArticles, 'edit')
    def edit(self, value, html):
        """ Update the properties """
        zLOG.LOG(self.meta_type, zLOG.INFO, 'edit')
        headers, body, format = handleText(text=value)
        self.value = str(body)
        self.html = utils._format_stx(text=self.value, level=self._stx_level)
        if format == 'stx':
            self.render_as = 'stx'
            self.size = len(self.html)
        else:
            self.render_as = 'html'
            self.size = len(self.value)


    security.declareProtected(ArticlePermissions.ChangeArticles, 'editExtra')
    def editExtra(self, alignment, clear, render_as):
        """ Update the properties """
        self.clear = str(clear)
        self.alignment = str(alignment)
        self.render_as = str(render_as)
        if self.render_as == 'stx':
            self.size = len(self.html)
        else:
            self.size = len(self.value)

#    security.declareProtected(CMFCorePermissions.View, 'getElementItems')
#    def getElementItems( self ):
#        """ Return a sequence of element items, used build the article """
#        return self.value is not '' and ((self.value,self.alignment),) or ()

    security.declareProtected(CMFCorePermissions.View, 'manage_FTPget')
    def manage_FTPget( self ):
        """ Return the text or html """
        return self.read()


    security.declareProtected(CMFCorePermissions.View, 'read')
    def read(self):
        """
        return either the raw text or the formatted text
        """
        if self.render_as == 'stx':
            return self.html
        else:
            return self.value

    def get_size(self):
        """
        return the size of the rendered value
        """
        return self.size

    def SearchableText(self):
        """
            SeachableText is used for full text searches of a portal.
            It should return a concatanation of all useful text.
        """
        return "%s %s" % (self.value, self.Description())

    def manage_afterAdd(self, item, container):
        """AbstractElement has an afterAdd method"""
        zLOG.LOG(self.meta_type, zLOG.INFO, 'manage_afterAdd(%s, %s)' % (item.getId(), container.getId()))
        AbstractElement.manage_afterAdd(self, item, container)


Globals.InitializeClass(TextElement)

# Register as an element type with the Article class
Article._elementTypes.append(TextElement)
