"""\
Declare interface for element classes, as used by Article
instances to build themselves.
"""
import Interface

class Element(Interface.Base):
    """\
    An Article is composed of Element objects. By supplying some basic information, the
    Element objects can be plugged into Articles without the Article
    having to be too aware of the Element types.
    """

    def Type(self):
        """\
        Return the type of element object this is (ie - 'Table Element')
        """

    def Description(self):
        """\
        Return a brief description of the element type.
        """

    def editableAttributes(self):
        """\
        Returns a tuble of editable attributes.  The values of this
        are used by the article to build commands to send to the
        'edit' method based on each element's setup.
        """

    def getEditForm(self):
        """\
        Return the name of a DTML component used to edit element.
        Editforms should be specific to their type of element.
        """

    def getView(self):
        """\
        Return the skinned name of the skin used to view the element.
        Views are specific to their type of element.
        """

    def apply(self, command):
        """\
        To make it easier to apply values from the rather dynamic
        Element edit form using Python Scripts, apply takes a
        mapping object as a default and applies itself to self.edit.

        It's basically a nice and protected wrapper around
        apply(self.edit, (), command).
        """

    def edit(self, **kw):
        """\
        The signature of this method should be specific to the
        element.  Using the values in the attribute
        '_editableAttributes', the Article can apply the right
        commands to each element object as its being edited without
        having to know too much about the structure.
        """

#    def elementItems(self):
#        """\
#        Return a sequence of key-value tuples, each representing
#        a value to be injected into the query dictionary (and,
#        therefore, tailored to work with the catalog).
#        """

    def SearchableText(self):
        """\
            SearchableText is used for full text searches of a portal.
            It should return a concatanation of all useful text.
        """
